#!/usr/bin/env node
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _panko = require('panko');

var _graphqlClient = require('graphql-client');

var _graphqlClient2 = _interopRequireDefault(_graphqlClient);

var _lodash = require('lodash.last');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { createSource } = _panko.sources;

const API_ENDPOINT = 'https://api.github.com/graphql';
const DEFAULT_BRANCH = 'master';
const QUERY = `
  query getDefinitions($after: String, $expression: String, $first: Int!, $login: String!) { 
    user(login: $login) {
      id
      repositories(first: $first, after: $after) {
        edges {
          cursor
          node {
            id
            object(expression: $expression) {
              ... on Blob {
                text
              }
            }
          }
        }
      }
    }
  }
`;

const getDefinitions = async (client, options) => {
  const { after, aggregated = [], branch, file, first = 100, handle } = options;

  const response = await client.query(QUERY, {
    after,
    expression: `${branch}:${file}`,
    first,
    login: handle
  }, (req, res) => {
    if (res.status === 401) throw new Error('Unauthorized. Please verify your handle and token.');
  });

  const { edges } = response.data.user.repositories;

  if (edges.length < 1) return aggregated;

  const definitions = edges.map(edge => edge.node.object && edge.node.object.text).map(JSON.parse).filter(Boolean);

  return getDefinitions(client, _extends({}, options, {
    after: (0, _lodash2.default)(edges).cursor,
    aggregated: aggregated.concat(definitions)
  }));
};

createSource(async options => {
  const client = (0, _graphqlClient2.default)({
    url: API_ENDPOINT,
    headers: {
      Authorization: `Bearer ${options.token}`
    }
  });

  return await getDefinitions(client, {
    file: options.file,
    branch: options.branch,
    handle: options.handle
  });
});