#!/usr/bin/env node

import { sources } from 'panko'
import createGraphQLClient from 'graphql-client'
import last from 'lodash.last'

const { createSource } = sources

const API_ENDPOINT = 'https://api.github.com/graphql'
const DEFAULT_BRANCH = 'master'
const QUERY = `
  query getDefinitions($after: String, $expression: String, $first: Int!, $login: String!) { 
    user(login: $login) {
      id
      repositories(first: $first, after: $after) {
        edges {
          cursor
          node {
            id
            object(expression: $expression) {
              ... on Blob {
                text
              }
            }
          }
        }
      }
    }
  }
`

const getDefinitions = async (client, options) => {
  const { after, aggregated = [], branch, file, first = 100, handle } = options

  const response = await client.query(
    QUERY,
    {
      after,
      expression: `${branch}:${file}`,
      first,
      login: handle,
    },
    (req, res) => {
      if (res.status === 401)
        throw new Error('Unauthorized. Please verify your handle and token.')
    }
  )

  const { edges } = response.data.user.repositories

  if (edges.length < 1) return aggregated

  const definitions = edges
    .map(edge => edge.node.object && edge.node.object.text)
    .map(JSON.parse)
    .filter(Boolean)

  return getDefinitions(client, {
    ...options,
    after: last(edges).cursor,
    aggregated: aggregated.concat(definitions),
  })
}

createSource(async options => {
  const client = createGraphQLClient({
    url: API_ENDPOINT,
    headers: {
      Authorization: `Bearer ${options.token}`,
    },
  })

  return await getDefinitions(client, {
    file: options.file,
    branch: options.branch,
    handle: options.handle,
  })
})
